import pygame
import sys
from random import randint
from game_manager.src.background import BackGround
from game_manager.utilities.draw import draw_line_dashed

class Tree(pygame.sprite.Sprite):
    def __init__(self,pos,group):
        super().__init__(group)
        self.image = pygame.image.load('graphics/tree.png').convert_alpha()
        self.rect = self.image.get_rect(topleft = pos)

class Player(pygame.sprite.Sprite):
    def __init__(self,pos,group):
        super().__init__(group)
        self.image = pygame.image.load('graphics/player.png').convert_alpha()
        self.rect = self.image.get_rect(center = pos)
        self.direction = pygame.math.Vector2()
        self.speed = 5

    def input(self):
        keys = pygame.key.get_pressed()

        if keys[pygame.K_UP]:
            self.direction.y = -1
        elif keys[pygame.K_DOWN]:
            self.direction.y = 1
        else:
            self.direction.y = 0

        if keys[pygame.K_RIGHT]:
            self.direction.x = 1
        elif keys[pygame.K_LEFT]:
            self.direction.x = -1
        else:
            self.direction.x = 0

    def update(self):
        self.input()
        self.rect.center += self.direction * self.speed

class CameraGroup(pygame.sprite.Group):
    def __init__(self, camera_types, SCREEN_HEIGHT, SCREEN_WIDTH):
        super().__init__()
        self.SCREEN_HEIGHT, self.SCREEN_WIDTH = SCREEN_HEIGHT, SCREEN_WIDTH
        self.display_surface = pygame.display.get_surface()

        # camera offset 
        self.offset = pygame.math.Vector2()
        self.half_w = self.display_surface.get_size()[0] // 2
        self.half_h = self.display_surface.get_size()[1] // 2

        self.camera_types = camera_types # types of the cameras

        # box setup
        self.camera_borders = {'left': 100, 'right': 100, 'top': 50, 'bottom': 50}
        
        l = self.camera_borders['left']
        t = self.camera_borders['top']

        w = self.display_surface.get_size()[0]  - (self.camera_borders['left'] + self.camera_borders['right'])
        h = self.display_surface.get_size()[1]  - (self.camera_borders['top'] + self.camera_borders['bottom'])
        
        self.camera_rect = pygame.Rect(l,t,w,h)
        self.mouse_offset_vector = pygame.math.Vector2((0, 0))
        # ground
        bg = BackGround()
        bg.from_image("graphics/ground.png")
        # self.ground_surf = pygame.image.load('graphics/ground.png').convert_alpha()
        self.ground_surf = bg.get_surface()
        self.ground_rect = self.ground_surf.get_rect(topleft = (0,0))

        # camera speed
        self.keyboard_speed = 5
        self.mouse_speed = 0.2

        # zoom 
        self.zoom_scale = 1
        self.internal_surf_size = (2500,2500)
        self.internal_surf = pygame.Surface(self.internal_surf_size, pygame.SRCALPHA)
        self.internal_rect = self.internal_surf.get_rect(center = (self.half_w,self.half_h))
        self.internal_surface_size_vector = pygame.math.Vector2(self.internal_surf_size)
        self.internal_offset = pygame.math.Vector2()
        self.internal_offset.x = self.internal_surf_size[0] // 2 - self.half_w
        self.internal_offset.y = self.internal_surf_size[1] // 2 - self.half_h

    def center_target_camera(self,target):
        self.offset.x = target.rect.centerx - self.half_w
        self.offset.y = target.rect.centery - self.half_h

    def box_target_camera(self,target):

        if target.rect.left < self.camera_rect.left:
            self.camera_rect.left = target.rect.left
        if target.rect.right > self.camera_rect.right:
            self.camera_rect.right = target.rect.right
        if target.rect.top < self.camera_rect.top:
            self.camera_rect.top = target.rect.top
        if target.rect.bottom > self.camera_rect.bottom:
            self.camera_rect.bottom = target.rect.bottom

        self.offset.x = self.camera_rect.left - self.camera_borders['left']
        self.offset.y = self.camera_rect.top - self.camera_borders['top']

    def keyboard_control(self):
        keys = pygame.key.get_pressed()
        if keys[pygame.K_a]: self.camera_rect.x -= self.keyboard_speed
        if keys[pygame.K_d]: self.camera_rect.x += self.keyboard_speed
        if keys[pygame.K_w]: self.camera_rect.y -= self.keyboard_speed
        if keys[pygame.K_s]: self.camera_rect.y += self.keyboard_speed

        self.offset.x = self.camera_rect.left - self.camera_borders['left']
        self.offset.y = self.camera_rect.top - self.camera_borders['top']

    def mouse_control(self):
        mouse = pygame.math.Vector2(pygame.mouse.get_pos())
        self.mouse_offset_vector = pygame.math.Vector2()

        left_border = self.camera_borders['left']
        top_border = self.camera_borders['top']
        right_border = self.display_surface.get_size()[0] - self.camera_borders['right']
        bottom_border = self.display_surface.get_size()[1] - self.camera_borders['bottom']

        if top_border < mouse.y < bottom_border:
            if mouse.x < left_border:
                self.mouse_offset_vector.x = mouse.x - left_border
                pygame.mouse.set_pos((left_border, mouse.y))
            if mouse.x > right_border:
                self.mouse_offset_vector.x = mouse.x - right_border
                pygame.mouse.set_pos((right_border, mouse.y))
        elif mouse.y < top_border:
            if mouse.x < left_border:
                self.mouse_offset_vector = mouse - pygame.math.Vector2(left_border, top_border)
                pygame.mouse.set_pos((left_border, top_border))
            if mouse.x > right_border:
                self.mouse_offset_vector = mouse - pygame.math.Vector2(right_border, top_border)
                pygame.mouse.set_pos((right_border,top_border))
        elif mouse.y > bottom_border:
            if mouse.x < left_border:
                self.mouse_offset_vector = mouse - pygame.math.Vector2(left_border, bottom_border)
                pygame.mouse.set_pos((left_border, bottom_border))
            if mouse.x > right_border:
                self.mouse_offset_vector = mouse - pygame.math.Vector2(right_border, bottom_border)
                pygame.mouse.set_pos((right_border, bottom_border))

        if left_border < mouse.x < right_border:
            if mouse.y < top_border:
                self.mouse_offset_vector.y = mouse.y - top_border
                pygame.mouse.set_pos((mouse.x, top_border))
            if mouse.y > bottom_border:
                self.mouse_offset_vector.y = mouse.y - bottom_border
                pygame.mouse.set_pos((mouse.x, bottom_border))

        self.offset += self.mouse_offset_vector * self.mouse_speed

    def zoom_keyboard_control(self):
        keys = pygame.key.get_pressed()
        if keys[pygame.K_q]:
            self.zoom_scale += 0.1
        if keys[pygame.K_e]:
            self.zoom_scale -= 0.1

    def camera_type(self, player):
        if "cneter_target" in self.camera_types:
            self.center_target_camera(player)
        
        if "box_target" in self.camera_types:
            self.box_target_camera(player)
        
        if "keyboard_control" in self.camera_types:
            self.keyboard_control()
        
        if "mouse_control" in self.camera_types:
            self.mouse_control()
        
        if "zoom_keyboard_control" in self.camera_types:
            self.zoom_keyboard_control()


    def draw_lines(self):
        
        draw_line_dashed(self.display_surface, (255, 0, 0), (self.camera_borders["left"], self.SCREEN_HEIGHT), (self.camera_borders["left"], 0), 2)

        # pygame.draw.line(self.display_surface, (255, 0, 0), [self.camera_borders["left"], self.SCREEN_HEIGHT], [ self.camera_borders["left"], 0], 2)
        draw_line_dashed(self.display_surface, (255, 0, 0), [self.SCREEN_WIDTH - self.camera_borders["right"],  self.SCREEN_WIDTH ], [self.SCREEN_WIDTH -self.camera_borders["right"], 0], 2)

        draw_line_dashed(self.display_surface, (255, 0, 0), [self.SCREEN_WIDTH, self.camera_borders["top"]], [0, self.camera_borders["top"]], 2)
        draw_line_dashed(self.display_surface, (255, 0, 0), [self.SCREEN_WIDTH, self.SCREEN_HEIGHT - self.camera_borders["bottom"]], [0, self.SCREEN_HEIGHT - self.camera_borders["bottom"]], 2)

    def custom_draw(self, player):
        self.camera_type(player) # run different camera controls

        self.internal_surf.fill('#71ddee')

        # ground 
        ground_offset = self.ground_rect.topleft - self.offset + self.internal_offset
        self.internal_surf.blit(self.ground_surf, ground_offset)

        # active elements
        for sprite in sorted(self.sprites(),key = lambda sprite: sprite.rect.centery):
            offset_pos = sprite.rect.topleft - self.offset + self.internal_offset
            self.internal_surf.blit(sprite.image, offset_pos)

        scaled_surf = pygame.transform.scale(self.internal_surf, self.internal_surface_size_vector * self.zoom_scale)
        scaled_rect = scaled_surf.get_rect(center = (self.half_w, self.half_h))

        self.display_surface.blit(scaled_surf, scaled_rect)

        self.draw_lines()

        xy = (player.rect.x - self.offset.x - self.mouse_offset_vector.x, player.rect.y - self.offset.y - self.mouse_offset_vector.y)
        
        player.text_label.render_text("Hi!", xy, self.display_surface, scale=self.zoom_scale, label_color=(255, 0, 0))


def main():
    pygame.init()
    screen = pygame.display.set_mode((1280,720))
    clock = pygame.time.Clock()
    pygame.event.set_grab(True)

    # setup 
    camera_group = CameraGroup()
    player = Player((640,360),camera_group)

    for i in range(20):
        random_x = randint(1000,2000)
        random_y = randint(1000,2000)
        Tree((random_x,random_y),camera_group)

    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    pygame.quit()
                    sys.exit()

            if event.type == pygame.MOUSEWHEEL:
                camera_group.zoom_scale += event.y * 0.03

        screen.fill('#71ddee')

        camera_group.update()
        camera_group.custom_draw(player)

        pygame.display.update()
        clock.tick(60)

if __name__ == "__main__":
    main()